#include <windows.h>

LRESULT APIENTRY WindowProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam); //Window events

struct MyBitmapHead
{
	BITMAPINFOHEADER bmiHeader; 
    RGBQUAD          bmiColors[2]; 
};

int APIENTRY WinMain(HINSTANCE hInst,HINSTANCE hPrev,LPSTR line,int CmdShow)
{
	//Create the main window
	HWND MainHwnd;
	WNDCLASS wc={CS_HREDRAW | CS_VREDRAW | CS_OWNDC, WindowProc, 0, 0, hInst, LoadIcon(hInst,NULL), LoadCursor(NULL,IDC_ARROW), (HBRUSH)GetStockObject(LTGRAY_BRUSH), NULL, "WhiteNoise"};
	RegisterClass(&wc);
	MainHwnd=CreateWindow("WhiteNoise",NULL,WS_POPUP | WS_MAXIMIZE,10,10,229,242,NULL,NULL,hInst,NULL);
	HDC MainDC=GetDC(MainHwnd);
	ShowWindow(MainHwnd,SW_SHOW);
	UpdateWindow(MainHwnd);

	//Calculate the window
	RECT WindowRect;
	DWORD FormWidth, FormHeight;
	GetWindowRect(MainHwnd, &WindowRect);
	FormWidth=WindowRect.right-WindowRect.left;
	FormHeight=WindowRect.bottom-WindowRect.top;

	//Display bitmap info
	const BitsPerByte=8;
	DWORD NumOfBytes=(FormWidth*FormHeight)/BitsPerByte;
	BYTE *RandomBits=new BYTE[NumOfBytes];
	MyBitmapHead TheBitmapInfo={{sizeof(BITMAPINFOHEADER), FormWidth, FormHeight, 1, 1, BI_RGB, 0, 0, 0, 2, 2}, {{0, 0, 0, 0x00}, {0xFF, 0xFF, 0xFF, 0x0}}}; //BITMAPINFOHEADER followed by 2 color indexes (black and white)

	//FPS
	DWORD StartTickCount=GetTickCount()-1000, TotalFrames=-1; //Counter variables - Get new FPS (of 0) immediately
	const FpsBufferWidth=32, FpsBufferHeight=20; //The buffers dimensions - width must be a multiple of BitsPerByte
	BYTE *FpsBuffer=new BYTE[FpsBufferWidth*FpsBufferHeight/BitsPerByte]; //A bit buffer in the same format as the DIB used to hold the visible FPS - This and following variables could be allocated and deallocated within the short FPS section below to save memory at the cost of time
	HDC FpsDC=CreateCompatibleDC(MainDC); //Used to get a visual representation of the FPS into the FpsBuffer
	HBITMAP FpsBitmap=CreateBitmap(FpsBufferWidth, FpsBufferHeight, 1, 1, NULL); //Used to get a visual representation of the FPS into the FpsBuffer
	SelectObject(FpsDC, FpsBitmap);
	SetBkColor(FpsDC, 0xFFFFFF); //Set background color to white

	//Main loop
	MSG msg;
	while(true)
    {
		//Check for windows message
        if(PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
        {
			if(GetMessage(&msg, NULL, NULL, NULL)==0)
				break;
            TranslateMessage(&msg);
            DispatchMessage(&msg);
			continue;
        }

		//Create static noise
		for(DWORD i=0;i<NumOfBytes;i++)
			RandomBits[i]=rand();

		//Copy FPS onto main screen
		//TextOut(MainDC, 10, 10, LastFramesChar, strlen(LastFramesChar));
		for(i=0;i<FpsBufferHeight;i++) //Copies one row at a time - The very first frame has a non initialized buffer, but it displays and goes by so quickly, it doesn't really matter
			memcpy(RandomBits+FormWidth/BitsPerByte*(FormHeight-i-1), FpsBuffer+FpsBufferWidth/BitsPerByte*i, FpsBufferWidth/BitsPerByte); //Assumes resolution width is divisible by BitsPerByte for bit packing
		
		//Copy to main form
		StretchDIBits(MainDC, 0, 0, FormWidth, FormHeight, 0, 0, FormWidth, FormHeight, RandomBits, (BITMAPINFO*)&TheBitmapInfo, DIB_RGB_COLORS, SRCCOPY);

		//FPS stuff
		TotalFrames++;
		if(GetTickCount()-StartTickCount>1000) //Every second update the FPS number
		{
			//Get FPS as string
			char LastFramesChar[5]; //Assumes FPS doesn't go above 9999
			itoa(TotalFrames, LastFramesChar, 10);

			//Get visual representation
			BitBlt(FpsDC, 0, 0, FpsBufferWidth, FpsBufferHeight, FpsDC, 0, 0, WHITENESS); //Clear to background (white) color
			TextOut(FpsDC, 0, 0, LastFramesChar, strlen(LastFramesChar));
			GetBitmapBits(FpsBitmap, FpsBufferWidth*FpsBufferHeight/BitsPerByte, FpsBuffer); //Couldn't use GetDIBits because it (and CreateDIBitmap and CreateDIBSection) never work properly
			//A real text width measurement could be done here with GetCharABCWidths, but it would probably be pointless since it would have to be 8 pixel aligned anyways

			//Reset FPS Counter
			TotalFrames=0;
			StartTickCount+=1000;
		}
    }

	//Release main handles and memory
	ReleaseDC(MainHwnd, MainDC);
	//DestroyWindow(MainHwnd); //Window should already be destroyed
	UnregisterClass("WhiteNoise", hInst);
	delete RandomBits;

	//Release FPS stuff
	delete[] FpsBuffer;
	DeleteObject(FpsBitmap);
	DeleteDC(FpsDC);

	//Return exit code
	return msg.wParam;
}

LRESULT APIENTRY WindowProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam) //Window events
{
	switch(msg)
	{
		case WM_DESTROY: //If window is destroyed, a key is pressed, or the window clicked, exit
		case WM_KEYDOWN:
		case WM_LBUTTONDOWN:
			PostQuitMessage(0);
			return 0;
	}	
	return DefWindowProc(hwnd,msg,wParam,lParam);
}
